@@ -5,7 +5,7 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.3.21
+# Version   : 0.3.22
 
 from operator import contains
 import  numpy                   as      np
@@ -125,6 +125,10 @@ class AFM:
         Function to align the contact point of the approach curve with zero by
         finding the first point to cross the axis and interpolating between this
         and the previous value. Note this method may be unstable for noisy curves.
+    adjust_tip_position:
+        Function to convert distance data from piezo position to tip position.
+    snap_in_distance:
+        Funciton to calculate distance from surface at which snap in begins.
     plot_adjusted:
         Function to plot individual, processed force curves. Curve number to
         plot must be given as an input <.plot_adjusted(n)>
@@ -213,6 +217,7 @@ class AFM:
         self.contact_point = []
         self.indent_depth = []
         self.snap_in = []
+        self.snap_dist = []
         self.max_force = []
 
         self.data_offset = []
@@ -664,7 +669,13 @@ class AFM:
                     self.indent_depth[i] = next(n for n in self.z_approach[:,i] if n == n)
 
             except:
-                print('WARNING: Could not locate contact point for curve number ', i, '. Consider checking baseline alignment.')
+                print('WARNING: Could not locate contact point for curve number ', i, '. Consider checking baseline alignment. Contact point from previous curve will be used instead.')
+
+                if np.isnan(intercept) == False:
+                    self.z_approach[:,i] = self.z_approach[:,i] - intercept
+                    self.z_withdraw[:,i] = self.z_withdraw[:,i] - intercept
+                    self.contact_point[i] = intercept
+                    self.indent_depth[i] = next(n for n in self.z_approach[:,i] if n == n)
 
     def adjust_tip_position(
         self
@@ -691,6 +702,52 @@ class AFM:
 
             self.indent_depth[i] = next(n for n in self.z_approach[:,i] if n == n)
 
+    def snap_in_distance(
+        self
+    ):
+        '''
+        Calculate distance from surface at which snap in begins.
+        '''
+
+        snap_distance = np.zeros(self.z_approach.shape[1])
+        
+        for i in range(0, self.z_approach.shape[1]):
+            f = self.approach[:,i]
+            d = self.z_approach[:,i]
+
+            snap_distance[i] = d[self.knee_point(f, d)]
+
+        self.snap_dist = snap_distance
+
+    def rolling_window(self,
+        a,
+        window
+    ):
+        shape = a.shape[:-1] + (a.shape[-1] - window + 1, window)
+        strides = a.strides + (a.strides[-1],)
+        return np.lib.stride_tricks.as_strided(a, shape=shape, strides=strides)
+
+    def knee_point(self,
+        X,
+        Y,
+        rolling_points = 5,
+        gradient_tolerance = 0.05
+    ):
+        grad = np.zeros([len(Y),1])
+
+        for i in range(0,len(Y)-11):
+            x = self.rolling_window(Y, rolling_points)[i]
+            y = self.rolling_window(X, rolling_points)[i]
+
+            if np.isnan(x).any():
+                continue
+
+            grad[i] = np.polyfit(x, y, deg = 1)[0]
+
+        grad = grad.flatten()
+
+        return len(grad) - next(x[0] for x in enumerate(np.flip(grad)) if x[1] > gradient_tolerance)
+
     def plot_adjusted(
         self,
         curve_number = 1,
@@ -878,6 +935,7 @@ class AFM:
             contact_point = self.contact_point,
             indent_depth = self.indent_depth,
             snap_in = self.snap_in,
+            snap_dist = self.snap_dist,
             max_force = self.max_force
         )
 
@@ -919,6 +977,7 @@ class AFM:
         self.contact_point = data['contact_point']
         self.indent_depth = data['indent_depth']
         self.snap_in = data['snap_in']
+        self.snap_dist = data['snap_dist']
         self.max_force = data['max_force']
 
     def calc_adhesion(
@@ -1452,17 +1511,21 @@ class AFM:
         topography_3d = False
     ):
 
-        plot_types = ['adhesion', 'modulus', 'topography', 'indent', 'max_force', 'snap_in']
+        plot_types = ['adhesion', 'modulus', 'topography', 'indent', 'max_force', 'snap_in','snap_dist']
         member = plot in plot_types
 
         if member == False:
-            print('Incorrect plot type specified. Available plots are adhesion, modulus, topography, indent, max_force and snap_in.')
+            print('Incorrect plot type specified. Available plots are adhesion, modulus, topography, indent, max_force, snap_in and snap_dist.')
 
         else:
 
+            _nid = ns.read(self.file_name)
+
             if auto_cell == True:
-                x_cells = int(np.sqrt(self.approach.shape[1]))
-                y_cells = x_cells
+                x_cells = _nid.param.SpecMap.maps[0][4]
+                y_cells = _nid.param.SpecMap.maps[0][5]
+                x_scale = np.asarray(_nid.param['Header Dump']['DataSet\Parameters\Imaging']['ScanRange'].split('[')[1].split(',')[0], dtype=float) * 1e6
+                y_scale = np.asarray(_nid.param['Header Dump']['DataSet\Parameters\Imaging']['ScanRange'].split('[')[1].split(',')[1], dtype=float) * 1e6
 
             x_pos = np.zeros(x_cells * y_cells)
             y_pos = np.zeros(x_cells * y_cells)
@@ -1472,6 +1535,7 @@ class AFM:
             depth = np.zeros(x_cells * y_cells)
             force = np.zeros(x_cells * y_cells)
             snap = np.zeros(x_cells * y_cells)
+            snap_dist  = np.zeros(x_cells * y_cells)
 
             x_counter = 1
             y_counter = 1
@@ -1500,6 +1564,9 @@ class AFM:
                 if plot == 'snap_in':
                     snap[j] = self.snap_in[j]
 
+                if plot == 'snap_dist':
+                    snap_dist[j] = self.snap_dist[j]
+
                 height[j] = self.contact_point[j]
 
                 loop_counter += 1
@@ -1530,6 +1597,9 @@ class AFM:
                     if plot == 'snap_in':
                         snap[j] = self.snap_in[j]
 
+                    if plot == 'snap_dist':
+                        snap_dist[j] = self.snap_dist[j]
+
                     height[j] = self.contact_point[j]
 
                     if updown == 1:
@@ -1572,6 +1642,9 @@ class AFM:
             elif plot == 'snap_in':
                 plt_title = 'Snap in force (nN)'
                 plt_data = snap
+            elif plot == 'snap_dist':
+                plt_title = 'Snap in distance (nm)'
+                plt_data = snap_dist
 
             if topography_3d == False:
 
@@ -1598,8 +1671,8 @@ class AFM:
                     showlegend = False,
                     width = 700, height = 700,
                     autosize = False,
-                    xaxis_title="X Position (um)",
-                    yaxis_title="Y Position (um)")
+                    xaxis_title="X Position ($\mu$m)",
+                    yaxis_title="Y Position ($\mu$m)")
 
                 fig.update_layout(
                     xaxis = dict(
@@ -1632,8 +1705,8 @@ class AFM:
                                 colorbar=dict(title=plt_title)))
 
                 fig.update_layout(scene = dict(
-                                xaxis_title='X Position (um)',
-                                yaxis_title='Y Position (um)',
+                                xaxis_title='X Position ($\mu$m)',
+                                yaxis_title='Y Position ($\mu$m)',
                                 zaxis_title='Z Position (nm)'),
                                 margin=dict(r=20, b=10, l=10, t=10))
 
